@@ -1,8 +1,15 @@
+import datetime
 import inspect
-from functools import wraps
+import warnings
+from functools import update_wrapper, wraps
+
+from dateutil.relativedelta import relativedelta
 
 from dagster import check
+from dagster.core.definitions.partition import PartitionSetDefinition
 from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
+from dagster.utils.backcompat import canonicalize_backcompat_args
+from dagster.utils.partitions import date_partition_range
 
 from ..decorator_utils import (
     InvalidDecoratedFunctionInfo,
@@ -11,14 +18,14 @@ from ..decorator_utils import (
     validate_decorated_fn_input_args,
     validate_decorated_fn_positionals,
 )
-from ..scheduler import Scheduler, SchedulerHandle
+from ..scheduler import SchedulerHandle
 from .composition import (
     InputMappingNode,
     composite_mapping_from_output,
     enter_composition,
     exit_composition,
 )
-from .config import ConfigMapping, resolve_config_field
+from .config import ConfigMapping
 from .events import ExpectationResult, Materialization, Output
 from .inference import (
     has_explicit_return_type,
@@ -28,7 +35,7 @@ from .inference import (
     infer_output_definitions,
 )
 from .input import InputDefinition
-from .mode import ModeDefinition
+from .mode import DEFAULT_MODE_NAME, ModeDefinition
 from .output import OutputDefinition
 from .pipeline import PipelineDefinition
 from .preset import PresetDefinition
@@ -66,7 +73,7 @@ class _LambdaSolid(object):
         positional_inputs = validate_solid_fn('@lambda_solid', self.name, fn, input_defs)
         compute_fn = _create_lambda_solid_compute_wrapper(fn, input_defs, output_def)
 
-        return SolidDefinition(
+        solid_def = SolidDefinition(
             name=self.name,
             input_defs=input_defs,
             output_defs=[output_def],
@@ -74,6 +81,8 @@ class _LambdaSolid(object):
             description=self.description,
             positional_inputs=positional_inputs,
         )
+        update_wrapper(solid_def, fn)
+        return solid_def
 
 
 class _Solid(object):
@@ -84,9 +93,8 @@ class _Solid(object):
         output_defs=None,
         description=None,
         required_resource_keys=None,
-        config_field=None,
-        metadata=None,
-        step_metadata_fn=None,
+        config=None,
+        tags=None,
     ):
         self.name = check.opt_str_param(name, 'name')
         self.input_defs = check.opt_nullable_list_param(input_defs, 'input_defs', InputDefinition)
@@ -99,13 +107,11 @@ class _Solid(object):
         # resources will be checked within SolidDefinition
         self.required_resource_keys = required_resource_keys
 
-        # config_field will be checked within SolidDefinition
-        self.config_field = config_field
+        # config will be checked within SolidDefinition
+        self.config = config
 
-        # metadata will be checked within ISolidDefinition
-        self.metadata = metadata
-
-        self.step_metadata_fn = check.opt_callable_param(step_metadata_fn, 'step_metadata_fn')
+        # tags will be checked within ISolidDefinition
+        self.tags = tags
 
     def __call__(self, fn):
         check.callable_param(fn, 'fn')
@@ -127,54 +133,26 @@ class _Solid(object):
         positional_inputs = validate_solid_fn('@solid', self.name, fn, input_defs, ['context'])
         compute_fn = _create_solid_compute_wrapper(fn, input_defs, output_defs)
 
-        return SolidDefinition(
+        solid_def = SolidDefinition(
             name=self.name,
             input_defs=input_defs,
             output_defs=output_defs,
             compute_fn=compute_fn,
-            config_field=self.config_field,
+            config=self.config,
             description=self.description,
             required_resource_keys=self.required_resource_keys,
-            metadata=self.metadata,
-            step_metadata_fn=self.step_metadata_fn,
+            tags=self.tags,
             positional_inputs=positional_inputs,
         )
+        update_wrapper(solid_def, fn)
+        return solid_def
 
 
-class _SchedulerHandle(object):
-    def __init__(self, scheduler_type):
-        self.scheduler_type = check.subclass_param(scheduler_type, 'scheduler_type', Scheduler)
-
-    def __call__(self, fn):
-        check.callable_param(fn, 'fn')
-
-        schedule_defs = fn()
-        for schedule_def in schedule_defs:
-            if not isinstance(schedule_def, ScheduleDefinition):
-                raise DagsterInvariantViolationError(
-                    '{fn_name} must return a list of ScheduleDefinitions'.format(
-                        fn_name=fn.__name__
-                    )
-                )
-
-        def handle_fn(artifacts_dir, repository_name):
-            return SchedulerHandle(
-                scheduler_type=self.scheduler_type,
-                schedule_defs=schedule_defs,
-                artifacts_dir=artifacts_dir,
-                repository_name=repository_name,
-            )
-
-        return handle_fn
-
-
-def schedules(scheduler):
+def schedules(name=None):
     '''Create a scheduler with a :py:class:`~dagster.core.scheduler.Scheduler` implementation and
     set of :py:class:`ScheduleDefinition` instances.
 
-    Decorate a function that returns a list of :py:class:`ScheduleDefinition`. The decorator has
-    an argument that takes a :py:class:`~dagster.core.scheduler.Scheduler` implementation, such as
-    :py:func:`~dagster_cron.SystemCronScheduler`.
+    Decorate a function that returns a list of :py:class:`ScheduleDefinition`.
 
     Args:
         scheduler (Scheduler): The scheduler implementation to use.
@@ -183,9 +161,7 @@ def schedules(scheduler):
 
     .. code-block:: python
 
-        from dagster_cron import SystemCronScheduler
-
-        @schedules(scheduler=SystemCronScheduler)
+        @schedules
         def define_scheduler():
             hello_world_schedule = ScheduleDefinition(
                 name='hello_world_schedule',
@@ -194,7 +170,12 @@ def schedules(scheduler):
 
             return [hello_world_schedule]
     '''
-    return _SchedulerHandle(scheduler)
+
+    if callable(name):
+        return SchedulerHandle(schedule_defs=name())
+
+    def _wrap(fn):
+        return SchedulerHandle(schedule_defs=fn())
 
 
 def lambda_solid(name=None, description=None, input_defs=None, output_def=None):
@@ -260,11 +241,10 @@ def solid(
     description=None,
     input_defs=None,
     output_defs=None,
-    config_field=None,
     config=None,
     required_resource_keys=None,
+    tags=None,
     metadata=None,
-    step_metadata_fn=None,
 ):
     '''Create a solid with the specified parameters from the decorated function.
 
@@ -278,11 +258,9 @@ def solid(
     The decorated function will be used as the solid's compute function. The signature of the
     decorated function is more flexible than that of the ``compute_fn`` in the core API; it may:
 
-    1. Return a value. This value will be wrapped in an :py:class:`Output` and yielded by the
-        compute function.
+    1. Return a value. This value will be wrapped in an :py:class:`Output` and yielded by the compute function.
     2. Return an :py:class:`Output`. This output will be yielded by the compute function.
-    3. Yield :py:class:`Output` or other `event objects <events>`_. Same as default compute
-        behaviour.
+    3. Yield :py:class:`Output` or other `event objects <events>`_. Same as default compute behaviour.
 
     Note that options 1) and 2) are incompatible with yielding other events -- if you would like
     to decorate a function that yields events, it must also wrap its eventual output in an
@@ -296,17 +274,29 @@ def solid(
             List of input definitions. Inferred from typehints if not provided.
         output_defs (Optional[List[OutputDefinition]]):
             List of output definitions. Inferred from typehints if not provided.
-        config (Optional[Dict[str, Field]]):
-            Defines the schema of the configuration dict provided to the solid via context.
-        config_field (Optional[Field]): Used in the rare case when a solid's top level config type
-            is other than a dictionary.
+        config (Optional[Any]): The schema for the config. Configuration data available
+            as context.solid_config. This value can be a:
 
-            A :class:`DagsterInvalidDefinitionError` will be raised if both ``config`` and
-            ``config_field`` are set.
+            1. A Python primitive type that resolve to dagster config
+               types: int, float, bool, str.
+
+            2. A dagster config type: Int, Float, Bool,
+               :py:class:`Array`, :py:class:`Noneable`, :py:class:`Selector`,
+               :py:class:`Shape`, :py:class:`Permissive`, etc
+
+            3. A bare python dictionary, which is wrapped in :py:class:`Shape`. Any
+               values in the dictionary get resolved by the same rules, recursively.
+
+            4. A bare python list of length one which itself is config type.
+               Becomes :py:class:`Array` with list element as an argument.
+
+            5. A instance of :py:class:`Field`.
         required_resource_keys (Optional[Set[str]]): Set of resource handles required by this solid.
-        metadata (Optional[Dict[Any, Any]]): Arbitrary metadata for the solid. Frameworks may
+        tags (Optional[Dict[str, Any]]): Arbitrary metadata for the solid. Frameworks may
             expect and require certain metadata to be attached to a solid. Users should generally
-            not set metadata directly.
+            not set metadata directly. Values that are not strings will be json encoded and must meet
+            the criteria that `json.loads(json.dumps(value)) == value`.
+
 
     Examples:
 
@@ -363,22 +353,21 @@ def solid(
         check.invariant(input_defs is None)
         check.invariant(output_defs is None)
         check.invariant(description is None)
-        check.invariant(config_field is None)
         check.invariant(config is None)
         check.invariant(required_resource_keys is None)
         check.invariant(metadata is None)
-        check.invariant(step_metadata_fn is None)
+        check.invariant(tags is None)
+
         return _Solid()(name)
 
     return _Solid(
         name=name,
         input_defs=input_defs,
         output_defs=output_defs,
-        config_field=resolve_config_field(config_field, config, '@solid'),
+        config=config,
         description=description,
         required_resource_keys=required_resource_keys,
-        metadata=metadata,
-        step_metadata_fn=step_metadata_fn,
+        tags=canonicalize_backcompat_args(tags, 'tags', metadata, 'metadata'),
     )
 
 
@@ -719,10 +708,25 @@ def composite_solid(
             :py:class:`CompositeSolidDefinition`.
 
             To map multiple outputs, return a dictionary from the composition function.
-        config (Optional[Dict[str, Field]]):
-            Defines the schema of the configuration dict provided to the composite solid via context.
-            If you specify ``config``, you must also specify ``config_fn``.
-        config_fn (Callable[[ConfigMappingContext, dict], dict]): By specifying a config mapping
+        config (Optional[Any]): The schema for the config. Must be combined with the `config_fn`
+            argument in order to transform this config into the config for the contained
+            solids.
+
+            1. A Python primitive type that resolve to dagster config
+               types: int, float, bool, str.
+
+            2. A dagster config type: Int, Float, Bool,
+               :py:class:`Array`, :py:class:`Noneable`, :py:class:`Selector`,
+               :py:class:`Shape`, :py:class:`Permissive`, etc
+
+            3. A bare python dictionary, which is wrapped in :py:class:`Shape`. Any
+               values in the dictionary get resolved by the same rules, recursively.
+
+            4. A bare python list of length one which itself is config type.
+               Becomes :py:class:`Array` with list element as an argument.
+
+            5. A instance of :py:class:`Field`.
+        config_fn (Callable[[dict], dict]): By specifying a config mapping
             function, you can override the configuration for the child solids contained within this
             composite solid.
 
@@ -766,7 +770,9 @@ def composite_solid(
 
 
 class _Pipeline(object):
-    def __init__(self, name=None, mode_defs=None, preset_defs=None, description=None):
+    def __init__(
+        self, name=None, mode_defs=None, preset_defs=None, description=None,
+    ):
         self.name = check.opt_str_param(name, 'name')
         self.mode_definitions = check.opt_list_param(mode_defs, 'mode_defs', ModeDefinition)
         self.preset_definitions = check.opt_list_param(preset_defs, 'preset_defs', PresetDefinition)
@@ -840,5 +846,224 @@ def pipeline(name=None, description=None, mode_defs=None, preset_defs=None):
         return _Pipeline()(name)
 
     return _Pipeline(
-        name=name, mode_defs=mode_defs, preset_defs=preset_defs, description=description
+        name=name, mode_defs=mode_defs, preset_defs=preset_defs, description=description,
     )
+
+
+def schedule(
+    name,
+    cron_schedule,
+    pipeline_name,
+    tags=None,
+    tags_fn=None,
+    solid_subset=None,
+    mode="default",
+    should_execute=None,
+    environment_vars=None,
+):
+    def inner(fn):
+        check.callable_param(fn, 'fn')
+
+        schedule_name = name or fn.__name__
+
+        return ScheduleDefinition(
+            name=schedule_name,
+            cron_schedule=cron_schedule,
+            pipeline_name=pipeline_name,
+            environment_dict_fn=fn,
+            tags=tags,
+            tags_fn=tags_fn,
+            solid_subset=solid_subset,
+            mode=mode,
+            should_execute=should_execute,
+            environment_vars=environment_vars,
+        )
+
+    return inner
+
+
+def monthly_schedule(
+    pipeline_name,
+    start_date,
+    name=None,
+    execution_day_of_month=1,
+    execution_time=datetime.time(0, 0),
+    tags_fn_for_date=None,
+    solid_subset=None,
+    mode="default",
+    should_execute=None,
+    environment_vars=None,
+):
+    check.opt_str_param(name, 'name')
+    check.inst_param(start_date, 'start_date', datetime.datetime)
+    check.opt_callable_param(tags_fn_for_date, 'tags_fn_for_date')
+    check.opt_nullable_list_param(solid_subset, 'solid_subset', of_type=str)
+    mode = check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME)
+    check.opt_callable_param(should_execute, 'should_execute')
+    check.opt_dict_param(environment_vars, 'environment_vars', key_type=str, value_type=str)
+    check.str_param(pipeline_name, 'pipeline_name')
+    check.inst_param(start_date, 'start_date', datetime.datetime)
+    check.int_param(execution_day_of_month, 'execution_day')
+    check.inst_param(execution_time, 'execution_time', datetime.time)
+
+    if execution_day_of_month <= 0 or execution_day_of_month > 31:
+        raise DagsterInvalidDefinitionError(
+            "`execution_day_of_month={}` is not valid for monthly schedule. Execution day must be between 1 and 31".format(
+                execution_day_of_month
+            )
+        )
+
+    cron_schedule = '{minute} {hour} {day} * *'.format(
+        minute=execution_time.minute, hour=execution_time.hour, day=execution_day_of_month
+    )
+
+    partition_fn = date_partition_range(start_date, delta=relativedelta(months=1), fmt="%Y-%m")
+
+    def inner(fn):
+        check.callable_param(fn, 'fn')
+
+        schedule_name = name or fn.__name__
+
+        tags_fn_for_partition_value = lambda partition: {}
+        if tags_fn_for_date:
+            tags_fn_for_partition_value = lambda partition: tags_fn_for_date(partition.value)
+
+        partition_set = PartitionSetDefinition(
+            name='{}_monthly'.format(pipeline_name),
+            pipeline_name=pipeline_name,
+            partition_fn=partition_fn,
+            environment_dict_fn_for_partition=lambda partition: fn(partition.value),
+            tags_fn_for_partition=tags_fn_for_partition_value,
+            mode=mode,
+        )
+
+        return partition_set.create_schedule_definition(
+            schedule_name,
+            cron_schedule,
+            should_execute=should_execute,
+            environment_vars=environment_vars,
+        )
+
+    return inner
+
+
+def daily_schedule(
+    pipeline_name,
+    start_date,
+    name=None,
+    execution_time=datetime.time(0, 0),
+    tags_fn_for_date=None,
+    solid_subset=None,
+    mode="default",
+    should_execute=None,
+    environment_vars=None,
+):
+    check.opt_str_param(name, 'name')
+    check.inst_param(start_date, 'start_date', datetime.datetime)
+    check.opt_callable_param(tags_fn_for_date, 'tags_fn_for_date')
+    check.opt_nullable_list_param(solid_subset, 'solid_subset', of_type=str)
+    mode = check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME)
+    check.opt_callable_param(should_execute, 'should_execute')
+    check.opt_dict_param(environment_vars, 'environment_vars', key_type=str, value_type=str)
+    check.str_param(pipeline_name, 'pipeline_name')
+    check.inst_param(start_date, 'start_date', datetime.datetime)
+    check.inst_param(execution_time, 'execution_time', datetime.time)
+
+    cron_schedule = '{minute} {hour} * * *'.format(
+        minute=execution_time.minute, hour=execution_time.hour
+    )
+
+    partition_fn = date_partition_range(start_date)
+
+    def inner(fn):
+        check.callable_param(fn, 'fn')
+
+        schedule_name = name or fn.__name__
+
+        tags_fn_for_partition_value = lambda partition: {}
+        if tags_fn_for_date:
+            tags_fn_for_partition_value = lambda partition: tags_fn_for_date(partition.value)
+
+        partition_set = PartitionSetDefinition(
+            name='{}_daily'.format(pipeline_name),
+            pipeline_name=pipeline_name,
+            partition_fn=partition_fn,
+            environment_dict_fn_for_partition=lambda partition: fn(partition.value),
+            tags_fn_for_partition=tags_fn_for_partition_value,
+            mode=mode,
+        )
+
+        return partition_set.create_schedule_definition(
+            schedule_name,
+            cron_schedule,
+            should_execute=should_execute,
+            environment_vars=environment_vars,
+        )
+
+    return inner
+
+
+def hourly_schedule(
+    pipeline_name,
+    start_date,
+    name=None,
+    execution_time=datetime.time(0, 0),
+    tags_fn_for_date=None,
+    solid_subset=None,
+    mode="default",
+    should_execute=None,
+    environment_vars=None,
+):
+    check.opt_str_param(name, 'name')
+    check.inst_param(start_date, 'start_date', datetime.datetime)
+    check.opt_callable_param(tags_fn_for_date, 'tags_fn_for_date')
+    check.opt_nullable_list_param(solid_subset, 'solid_subset', of_type=str)
+    mode = check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME)
+    check.opt_callable_param(should_execute, 'should_execute')
+    check.opt_dict_param(environment_vars, 'environment_vars', key_type=str, value_type=str)
+    check.str_param(pipeline_name, 'pipeline_name')
+    check.inst_param(start_date, 'start_date', datetime.datetime)
+    check.inst_param(execution_time, 'execution_time', datetime.time)
+
+    if execution_time.hour != 0:
+        warnings.warn(
+            "Hourly schedule {schedule_name} created with:\n"
+            "\tschedule_time=datetime.time(hour={hour}, minute={minute}, ...)."
+            "Since this is a hourly schedule, the hour parameter will be ignored and the schedule "
+            "will run on the {minute} mark for the previous hour interval. Replace "
+            "datetime.time(hour={hour}, minute={minute}, ...) with datetime.time(minute={minute}, ...)"
+            "to fix this warning."
+        )
+
+    cron_schedule = '{minute} * * * *'.format(minute=execution_time.minute)
+
+    partition_fn = date_partition_range(
+        start_date, delta=datetime.timedelta(hours=1), fmt="%Y-%m-%d-%H:%M"
+    )
+
+    def inner(fn):
+        check.callable_param(fn, 'fn')
+
+        schedule_name = name or fn.__name__
+
+        tags_fn_for_partition_value = lambda partition: {}
+        if tags_fn_for_date:
+            tags_fn_for_partition_value = lambda partition: tags_fn_for_date(partition.value)
+
+        partition_set = PartitionSetDefinition(
+            name='{}_hourly'.format(pipeline_name),
+            pipeline_name=pipeline_name,
+            partition_fn=partition_fn,
+            environment_dict_fn_for_partition=lambda partition: fn(partition.value),
+            tags_fn_for_partition=tags_fn_for_partition_value,
+            mode=mode,
+        )
+
+        return partition_set.create_schedule_definition(
+            schedule_name,
+            cron_schedule,
+            should_execute=should_execute,
+            environment_vars=environment_vars,
+        )
+
+    return inner
