@@ -3,7 +3,7 @@ from collections import namedtuple
 from dagster import check
 from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
 from dagster.core.serdes import whitelist_for_serdes
-from dagster.core.types.runtime import construct_runtime_type_dictionary
+from dagster.core.types.dagster_type import construct_dagster_type_dictionary
 
 from .container import IContainSolids, create_execution_structure, validate_dependency_dict
 from .dependency import (
@@ -80,6 +80,8 @@ class PipelineDefinition(IContainSolids, object):
             to ship common combinations of options to pipeline end users in Python code, and can
             be selected by tools like Dagit.
 
+        _parent_pipeline_def (INTERNAL ONLY): Used for tracking pipelines created using solid subsets.
+
     Examples:
 
         .. code-block:: python
@@ -89,11 +91,11 @@ class PipelineDefinition(IContainSolids, object):
                 return 1
 
 
-            @solid(input_defs=[InputDefinition('num')])
+            @solid(input_defs=[InputDefinition('num')], required_resource_keys={'op'})
             def apply_op(context, num):
                 return context.resources.op(num)
 
-            @resource(config_field=Field(Int))
+            @resource(config=Int)
             def adder_resource(init_context):
                 return lambda x: x + init_context.resource_config
 
@@ -129,6 +131,7 @@ class PipelineDefinition(IContainSolids, object):
         dependencies=None,
         mode_defs=None,
         preset_defs=None,
+        _parent_pipeline_def=None,  # https://github.com/dagster-io/dagster/issues/2115
     ):
         self._name = check.opt_str_param(name, 'name', '<<unnamed>>')
         self._description = check.opt_str_param(description, 'description')
@@ -164,7 +167,7 @@ class PipelineDefinition(IContainSolids, object):
         self._solid_dict = solid_dict
         self._dependency_structure = dependency_structure
 
-        self._runtime_type_dict = construct_runtime_type_dictionary(self._current_level_solid_defs)
+        self._runtime_type_dict = construct_dagster_type_dictionary(self._current_level_solid_defs)
 
         self._preset_defs = check.opt_list_param(preset_defs, 'preset_defs', PresetDefinition)
         self._preset_dict = {}
@@ -192,9 +195,9 @@ class PipelineDefinition(IContainSolids, object):
         _validate_inputs(self._dependency_structure, self._solid_dict)
 
         self._all_solid_defs = _build_all_solid_defs(self._current_level_solid_defs)
-
-        self._selector = ExecutionSelector(self.name, list(solid_dict.keys()))
-
+        self._parent_pipeline_def = check.opt_inst_param(
+            _parent_pipeline_def, '_parent_pipeline_def', PipelineDefinition
+        )
         self._cached_enviroment_schemas = {}
 
     def get_environment_schema(self, mode=None):
@@ -349,7 +352,10 @@ class PipelineDefinition(IContainSolids, object):
 
     @property
     def selector(self):
-        return self._selector
+        if self._parent_pipeline_def is None:
+            return ExecutionSelector(self.name)
+
+        return ExecutionSelector(self.name, list(self._solid_dict.keys()))
 
     def has_runtime_type(self, name):
         check.str_param(name, 'name')
@@ -381,6 +387,7 @@ class PipelineDefinition(IContainSolids, object):
         return name in self._all_solid_defs
 
     def build_sub_pipeline(self, solid_subset):
+        check.opt_list_param(solid_subset, 'solid_subset', of_type=str)
         return self if solid_subset is None else _build_sub_pipeline(self, solid_subset)
 
     def get_presets(self):
@@ -459,6 +466,7 @@ def _build_sub_pipeline(pipeline_def, solid_names):
         solid_defs=list({solid.definition for solid in solids}),
         mode_defs=pipeline_def.mode_definitions,
         dependencies=deps,
+        _parent_pipeline_def=pipeline_def,
     )
     handle, _ = ExecutionTargetHandle.get_handle(pipeline_def)
     if handle:
@@ -476,16 +484,16 @@ def _validate_resource_dependencies(mode_definitions, solid_defs):
 
     for mode_def in mode_definitions:
         mode_resources = set(mode_def.resource_defs.keys())
-        for solid in solid_defs:
-            for required_resource in solid.required_resource_keys:
+        for solid_def in solid_defs:
+            for required_resource in solid_def.required_resource_keys:
                 if required_resource not in mode_resources:
                     raise DagsterInvalidDefinitionError(
                         (
-                            'Resource "{resource}" is required by solid {solid_name}, but is not '
+                            'Resource "{resource}" is required by solid def {solid_def_name}, but is not '
                             'provided by mode "{mode_name}".'
                         ).format(
                             resource=required_resource,
-                            solid_name=solid.name,
+                            solid_def_name=solid_def.name,
                             mode_name=mode_def.name,
                         )
                     )
@@ -516,7 +524,7 @@ def _validate_inputs(dependency_structure, solid_dict):
                         'Input "{input_name}" in solid "{solid_name}" is not connected to '
                         'the output of a previous solid and can not be hydrated from configuration, '
                         'creating an impossible to execute pipeline. '
-                        'Posible solutions are:\n'
+                        'Possible solutions are:\n'
                         '  * add a input_hydration_config for the type "{runtime_type}"\n'
                         '  * connect "{input_name}" to the output of another solid\n'.format(
                             solid_name=solid.name,
@@ -559,19 +567,24 @@ class ExecutionSelector(namedtuple('_ExecutionSelector', 'name solid_subset')):
 
 
 @whitelist_for_serdes
-class PipelineRunsFilter(
-    namedtuple('_PipelineRunsFilter', 'run_id tag_key tag_value pipeline status')
-):
-    def __new__(cls, run_id=None, pipeline=None, status=None, tag_key=None, tag_value=None):
+class PipelineRunsFilter(namedtuple('_PipelineRunsFilter', 'run_id tags pipeline_name status')):
+    def __new__(cls, run_id=None, pipeline_name=None, status=None, tags=None):
         return super(PipelineRunsFilter, cls).__new__(
             cls,
             run_id=check.opt_str_param(run_id, 'run_id'),
-            tag_key=check.opt_str_param(tag_key, 'tag_key'),
-            tag_value=check.opt_str_param(tag_value, 'tag_value'),
-            pipeline=check.opt_str_param(pipeline, 'pipeline'),
+            tags=check.opt_dict_param(tags, 'tags', key_type=str, value_type=str),
+            pipeline_name=check.opt_str_param(pipeline_name, 'pipeline_name'),
             status=status,
         )
 
+    def to_graphql_input(self):
+        return {
+            'runId': self.run_id,
+            'tags': [{'key': k, 'value': v} for k, v in self.tags.items()],
+            'pipelineName': self.pipeline_name,
+            'status': self.status,
+        }
+
 
 def _create_environment_schema(pipeline_def, mode_definition):
     from .environment_configs import (
@@ -581,7 +594,7 @@ def _create_environment_schema(pipeline_def, mode_definition):
     )
     from .environment_schema import EnvironmentSchema
 
-    environment_cls = define_environment_cls(
+    environment_type = define_environment_cls(
         EnvironmentClassCreationData(
             pipeline_name=pipeline_def.name,
             solids=pipeline_def.solids,
@@ -591,8 +604,6 @@ def _create_environment_schema(pipeline_def, mode_definition):
         )
     )
 
-    environment_type = environment_cls.inst()
-
     config_type_dict_by_name, config_type_dict_by_key = construct_config_type_dictionary(
         pipeline_def.all_solid_defs, environment_type
     )
