@@ -1,10 +1,23 @@
-import warnings
 from collections import namedtuple
 
 from dagster import check
 from dagster.core.errors import DagsterInvalidDefinitionError
+from dagster.core.instance import DagsterInstance
 from dagster.core.serdes import whitelist_for_serdes
 
+from .mode import DEFAULT_MODE_NAME
+
+
+class ScheduleExecutionContext:
+    ''' Defines the context where an instance is configured to execute scheduled jobs '''
+
+    def __init__(self, instance):
+        self._instance = check.inst_param(instance, 'instance', DagsterInstance)
+
+    @property
+    def instance(self):
+        return self._instance
+
 
 @whitelist_for_serdes
 class ScheduleDefinitionData(
@@ -28,17 +41,22 @@ class ScheduleDefinition(object):
         pipeline_name (str): The name of the pipeline definition
         environment_dict (Optional[dict]): (deprecated) The environment config that parameterizes
             this execution, as a dict.
-        environment_dict_fn (Callable[[DagsterInstance], [Dict]]): A function that takes a
-            DagsterInstance and returns the environment configuration that parameterizes this
-            execution, as a dict.
+        environment_dict_fn (Callable[ScheduleExecutionContext, [Dict]]): A function that takes a
+            ScheduleExecutionContext object and returns the environment configuration that
+            parameterizes this execution, as a dict.
         tags (Optional[dict[str, str]]]): (deprecated) A dictionary of tags (key value pairs) that
             will be added to the generated run.
-        tags_fn (Callable[[DagsterInstance], Optional[dict[str, str]]]): A function that returns a
-            dictionary of tags (key value pairs) that will be added to the generated run.
-        should_execute (Optional[function]): Function that runs at schedule execution time that
-            determines whether a schedule should execute. Defaults to a function that always returns
-            ``True``.
-        environment_vars (Optional[dict]): The environment variables to set for the schedule
+        tags_fn (Callable[ScheduleExecutionContext, Optional[dict[str, str]]]): A function that
+            takes a ScheduleExecutionContext object and returns a dictionary of tags (key value
+            pairs) that will be added to the generated run.
+        solid_subset (Optional[List[str]]): The list of names of solid invocations (i.e., of
+            unaliased solids or of their aliases if aliased) to execute with this schedule.
+        mode (Optional[str]): The mode to apply when executing this schedule. (default: 'default')
+        should_execute (Optional[Callable[ScheduleExecutionContext, bool]]): Function that takes a
+            ScheduleExecutionContext object and runs at schedule execution time that determines
+            whether a schedule should execute. Defaults to a function that always returns ``True``.
+        environment_vars (Optional[dict[str, str]]): The environment variables to set for the
+            schedule
     '''
 
     __slots__ = [
@@ -49,7 +67,6 @@ class ScheduleDefinition(object):
         '_tags',
         '_tags_fn',
         '_should_execute',
-        '_partition_selector',
     ]
 
     def __init__(
@@ -61,35 +78,22 @@ class ScheduleDefinition(object):
         environment_dict_fn=None,
         tags=None,
         tags_fn=None,
+        solid_subset=None,
         mode="default",
-        should_execute=lambda: True,
+        should_execute=None,
         environment_vars=None,
     ):
+        check.str_param(name, 'name')
+        check.str_param(cron_schedule, 'cron_schedule')
         check.str_param(pipeline_name, 'pipeline_name')
-        check.str_param(mode, 'mode')
         check.opt_dict_param(environment_dict, 'environment_dict')
         check.opt_callable_param(environment_dict_fn, 'environment_dict_fn')
         check.opt_dict_param(tags, 'tags', key_type=str, value_type=str)
         check.opt_callable_param(tags_fn, 'tags_fn')
-        check.callable_param(should_execute, 'should_execute')
-
-        if environment_dict:
-            warnings.warn(
-                "The `environment_dict` argument to `ScheduleDefinition` is deprecated. "
-                "Use the `environment_dict_fn` argument instead. "
-                "The argument `environment_dict_fn` should be a function that takes `DagsterInstance` "
-                "as an argument and returns an environment dict",
-                DeprecationWarning,
-            )
-
-        if tags:
-            warnings.warn(
-                "The `tags` argument to `ScheduleDefinition` is deprecated. "
-                "Use the `tags_fn` argument instead. "
-                "The argument `tags_fn` should be a function that takes DagsterInstance as an "
-                "argument and returns the dictionary of tags",
-                DeprecationWarning,
-            )
+        check.opt_nullable_list_param(solid_subset, 'solid_subset', of_type=str)
+        mode = check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME)
+        check.opt_callable_param(should_execute, 'should_execute')
+        check.opt_dict_param(environment_vars, 'environment_vars', key_type=str, value_type=str)
 
         if environment_dict_fn and environment_dict:
             raise DagsterInvalidDefinitionError(
@@ -104,10 +108,13 @@ class ScheduleDefinition(object):
             )
 
         if not environment_dict and not environment_dict_fn:
-            environment_dict_fn = lambda: {}
+            environment_dict_fn = lambda _context: {}
 
         if not tags and not tags_fn:
-            tags_fn = lambda: {}
+            tags_fn = lambda _context: {}
+
+        if not should_execute:
+            should_execute = lambda _context: True
 
         self._schedule_definition_data = ScheduleDefinitionData(
             name=check.str_param(name, 'name'),
@@ -147,22 +154,18 @@ class ScheduleDefinition(object):
     def execution_params(self):
         return self._execution_params
 
-    @property
-    def environment_dict(self):
-        return self._environment_dict
+    def get_environment_dict(self, context):
+        check.inst_param(context, 'context', ScheduleExecutionContext)
+        if self._environment_dict:
+            return self._environment_dict
+        return self._environment_dict_fn(context)
 
-    @property
-    def environment_dict_fn(self):
-        return self._environment_dict_fn
+    def get_tags(self, context):
+        check.inst_param(context, 'context', ScheduleExecutionContext)
+        if self._tags:
+            return self._tags
+        return self._tags_fn(context)
 
-    @property
-    def tags(self):
-        return self._tags
-
-    @property
-    def tags_fn(self):
-        return self._tags_fn
-
-    @property
-    def should_execute(self):
-        return self._should_execute
+    def should_execute(self, context):
+        check.inst_param(context, 'context', ScheduleExecutionContext)
+        return self._should_execute(context)
