@@ -37,6 +37,51 @@ class DagsterInvalidDefinitionError(DagsterError):
     '''Indicates that the rules for a definition have been violated by the user.'''
 
 
+CONFIG_ERROR_VERBIAGE = '''
+This value can be a:
+    - Field
+    - Python primitive types that resolve to dagster config types
+        - int, float, bool, str, list.
+    - A dagster config type: Int, Float, Bool, List, Optional, Selector, Dict
+    - A bare python dictionary, which is wrapped in Field(Dict(...)). Any values
+      in the dictionary get resolved by the same rules, recursively.
+    - A python list with a single entry that can resolve to a type, e.g. [int]
+'''
+
+
+class DagsterInvalidConfigDefinitionError(DagsterError):
+    '''Indicates that you have attempted to construct a config with an invalid value
+
+    This value can be a:
+
+        - :py:class:`Field`
+        - Python primitive types that resolve to dagster config types
+            - int, float, bool, str, list.
+        - A dagster config type: Int, Float, Bool, List, Optional, :py:class:`Selector`, :py:class:`Dict`
+        - A bare python dictionary, which is wrapped in Field(Dict(...)). Any values
+          in the dictionary get resolved by the same rules, recursively.
+        - A python list with a single entry that can resolve to a type, e.g. [int]
+    '''
+
+    def __init__(self, original_root, current_value, stack, reason=None, **kwargs):
+        self.original_root = original_root
+        self.current_value = current_value
+        self.stack = stack
+        super(DagsterInvalidConfigDefinitionError, self).__init__(
+            (
+                'Error defining config. Original value passed: {original_root}. '
+                '{stack_str}{current_value} '
+                'cannot be resolved.{reason_str}' + CONFIG_ERROR_VERBIAGE
+            ).format(
+                original_root=repr(original_root),
+                stack_str='Error at stack path :' + ':'.join(stack) + '. ' if stack else '',
+                current_value=repr(current_value),
+                reason_str=' Reason: {reason}.'.format(reason=reason) if reason else '',
+            ),
+            **kwargs
+        )
+
+
 class DagsterInvariantViolationError(DagsterError):
     '''Indicates the user has violated a well-defined invariant that can only be enforced
     at runtime.'''
@@ -139,8 +184,7 @@ class DagsterUserCodeExecutionError(DagsterError):
         user_exception = check.inst_param(kwargs.pop('user_exception'), 'user_exception', Exception)
         original_exc_info = check.tuple_param(kwargs.pop('original_exc_info'), 'original_exc_info')
 
-        if original_exc_info[0] is None:
-            raise Exception('bad dude {}'.format(type(self)))
+        check.invariant(original_exc_info[0] is not None)
 
         msg = _add_inner_exception_for_py2(args[0], original_exc_info)
 
@@ -194,24 +238,63 @@ class DagsterResourceFunctionError(DagsterUserCodeExecutionError):
     '''
 
 
+class DagsterConfigMappingFunctionError(DagsterUserCodeExecutionError):
+    '''
+    Indicates that an unexpected error occured while executing the body of a config mapping
+    function defined in a :class:`CompositeSolidDefinition <dagster.CompositeSolidDefinition>`
+    during config parsing.
+    '''
+
+
+class DagsterInputHydrationConfigError(DagsterUserCodeExecutionError):
+    '''
+    Indicates that an unexpected error occurred while executing the body of an input hydration
+    config function defined in a :class:`InputHydrationConfig <dagster.InputHydrationConfig>` during
+    input hydration of a custom type
+    '''
+
+
+class DagsterOutputMaterializationError(DagsterUserCodeExecutionError):
+    '''
+    Indicates that an unexpected error occurred while executing the body of an output
+    materialization function defined in a
+    :class:`OutputMaterializationConfig <dagster.OutputMaterializationConfig>` during output
+    materialization of a custom type
+    '''
+
+
+class DagsterUnknownResourceError(DagsterError, AttributeError):
+    # inherits from AttributeError as it is raised within a __getattr__ call... used to support
+    # object hasattr method
+    ''' Indicates that an unknown resource was accessed in the body of an execution step. May often
+    happen by accessing a resource in the compute function of a solid without first supplying the
+    solid with the correct `required_resource_keys` argument.
+    '''
+
+    def __init__(self, resource_name, *args, **kwargs):
+        self.resource_name = check.str_param(resource_name, 'resource_name')
+        msg = (
+            'Unknown resource `{resource_name}. Specify `{resource_name}` as a required resource '
+            'on the compute / config function that accessed it.'
+        ).format(resource_name=resource_name)
+        super(DagsterUnknownResourceError, self).__init__(msg, *args, **kwargs)
+
+
 class DagsterInvalidConfigError(DagsterError):
     '''Thrown when provided config is invalid (does not type check against the relevant config
     schema).'''
 
-    def __init__(self, pipeline, errors, config_value, *args, **kwargs):
-        from dagster.core.definitions import PipelineDefinition
-        from dagster.core.types.evaluator.errors import friendly_string_for_error, EvaluationError
+    def __init__(self, preamble, errors, config_value, *args, **kwargs):
+        from dagster.config.errors import (
+            friendly_string_for_error,
+            EvaluationError,
+        )
 
-        self.pipeline = check.opt_inst_param(pipeline, 'pipeline', PipelineDefinition)
+        check.str_param(preamble, 'preamble')
         self.errors = check.list_param(errors, 'errors', of_type=EvaluationError)
-
         self.config_value = config_value
 
-        if pipeline is not None:
-            error_msg = 'Pipeline "{pipeline}" config errors:'.format(pipeline=pipeline.name)
-        else:
-            error_msg = 'Config errors:'
-
+        error_msg = preamble
         error_messages = []
 
         for i_error, error in enumerate(self.errors):
@@ -272,3 +355,7 @@ class DagsterInstanceMigrationRequired(DagsterError):
 
 class DagsterRunAlreadyExists(DagsterError):
     '''Indicates that a pipeline run already exists in a run storage.'''
+
+
+class DagsterRunConflict(DagsterError):
+    '''Indicates that a conflicting pipeline run exists in a run storage.'''
