@@ -107,11 +107,7 @@ def _unpack_value(val, enum_map, tuple_map):
         # The call to seven.get_args turns out to be pretty expensive -- we should probably turn
         # to, e.g., manually managing the deprecated keys on the serdes constructor.
         args_for_class = seven.get_args(klass)
-        filtered_val = (
-            {k: v for k, v in unpacked_val.items() if k in args_for_class}
-            if args_for_class
-            else unpacked_val
-        )
+        filtered_val = {k: v for k, v in unpacked_val.items() if k in args_for_class}
         return klass(**filtered_val)
     if isinstance(val, dict) and val.get('__enum__'):
         name, member = val['__enum__'].split('.')
@@ -141,7 +137,10 @@ class ConfigurableClassData(
     '''Serializable tuple describing where to find a class and the config fragment that should
     be used to instantiate it.
 
-    Classes serialized in this way should implement dagster.serdes.ConfigurableClass.
+    Users should not instantiate this class directly.
+    
+    Classes intended to be serialized in this way should implement the 
+    :py:class:`dagster.serdes.ConfigurableClass` mixin.
     '''
 
     def __new__(cls, module_name, class_name, config_yaml):
@@ -162,9 +161,10 @@ class ConfigurableClassData(
             )
         )
 
-    def rehydrate(self, **constructor_kwargs):
+    def rehydrate(self):
         from dagster.core.errors import DagsterInvalidConfigError
-        from dagster.core.types.evaluator import evaluate_config
+        from dagster.config.field import resolve_to_config_type
+        from dagster.config.validate import process_config
 
         try:
             module = importlib.import_module(self.module_name)
@@ -196,11 +196,15 @@ class ConfigurableClassData(
                 ConfigurableClass,
             )
 
-        config_dict = yaml.load(self.config_yaml)
-        result = evaluate_config(klass.config_type().inst(), config_dict)
+        config_dict = yaml.safe_load(self.config_yaml)
+        result = process_config(resolve_to_config_type(klass.config_type()), config_dict)
         if not result.success:
-            raise DagsterInvalidConfigError(None, result.errors, config_dict)
-        return klass.from_config_value(self, result.value, **constructor_kwargs)
+            raise DagsterInvalidConfigError(
+                'Errors whilst loading configuration for {}.'.format(klass.config_type()),
+                result.errors,
+                config_dict,
+            )
+        return klass.from_config_value(self, result.value)
 
 
 class ConfigurableClass(six.with_metaclass(ABCMeta)):
@@ -211,12 +215,14 @@ class ConfigurableClass(six.with_metaclass(ABCMeta)):
     file in which third parties can place plugin packages. Instead, the intention is to make, e.g.,
     run storage, pluggable with a config chunk like:
 
+    .. code-block:: yaml
+
         run_storage:
             module: very_cool_package.run_storage
             class: SplendidRunStorage
             config:
                 magic_word: "quux"
-    
+
     This same pattern should eventually be viable for other system components, e.g. engines.
 
     The ConfigurableClass mixin provides the necessary hooks for classes to be instantiated from
@@ -225,10 +231,10 @@ class ConfigurableClass(six.with_metaclass(ABCMeta)):
     Pieces of the Dagster system which we wish to make pluggable in this way should consume a config
     type such as:
 
-        SystemNamedDict(
-            name,
-            {'module': Field(String), 'class': Field(String), 'config': Field(PermissiveDict())},
-        )
+    .. code-block:: python
+
+        {'module': str, 'class': str, 'config': Field(Permissive())}
+
     '''
 
     @abstractproperty
@@ -243,27 +249,27 @@ class ConfigurableClass(six.with_metaclass(ABCMeta)):
     def config_type(cls):
         '''dagster.ConfigType: The config type against which to validate a config yaml fragment
         serialized in an instance of ConfigurableClassData.
-        
-        This is usually an instance of dagster.core.definitions.environment_configs.SystemNamedDict.
         '''
 
     @staticmethod
     @abstractmethod
-    def from_config_value(inst_data, config_value, **kwargs):
+    def from_config_value(inst_data, config_value):
         '''New up an instance of the ConfigurableClass from a validated config value.
 
         Called by ConfigurableClassData.rehydrate.
-        
+
         Args:
             config_value (dict): The validated config value to use. Typically this should be the
                 `value` attribute of a dagster.core.types.evaluator.evaluation.EvaluateValueResult.
 
 
-        A common pattern is for the implementation to splat the config_value with the kwargs, to
-        align with the signature of the ConfigurableClass's constructor and allow overrides:
+        A common pattern is for the implementation to align the config_value with the signature
+        of the ConfigurableClass's constructor:
+
+        .. code-block:: python
 
             @staticmethod
-            def from_config_value(inst_data, config_value, **kwargs):
-                return MyConfigurableClass(inst_data=inst_data, **dict(config_value, **kwargs))
+            def from_config_value(inst_data, config_value):
+                return MyConfigurableClass(inst_data=inst_data, **config_value)
 
         '''
