@@ -12,6 +12,7 @@ import tempfile
 import threading
 from collections import namedtuple
 from enum import Enum
+from warnings import warn
 
 import yaml
 from six.moves import configparser
@@ -21,7 +22,6 @@ from dagster.core.errors import DagsterInvariantViolationError
 from dagster.seven import IS_WINDOWS, thread
 from dagster.seven.abc import Mapping
 
-from .subprocess_pdb import ForkedPdb
 from .yaml_utils import load_yaml_from_glob_list, load_yaml_from_globs, load_yaml_from_path
 
 if sys.version_info > (3,):
@@ -298,9 +298,6 @@ def touch_file(path):
         os.utime(path, None)
 
 
-pdb = ForkedPdb()
-
-
 def _kill_on_event(termination_event):
     termination_event.wait()
     if IS_WINDOWS:
@@ -333,3 +330,21 @@ def start_termination_thread(termination_event):
 def datetime_as_float(dt):
     check.inst_param(dt, 'dt', datetime.datetime)
     return float((dt - EPOCH).total_seconds())
+
+
+# hashable frozen string to string dict
+class frozentags(frozendict):
+    def __init__(self, *args, **kwargs):
+        super(frozentags, self).__init__(*args, **kwargs)
+        check.dict_param(self, 'self', key_type=str, value_type=str)
+
+    def __hash__(self):
+        return hash(tuple(sorted(self.items())))
+
+    def updated_with(self, new_tags):
+        check.dict_param(new_tags, 'new_tags', key_type=str, value_type=str)
+        updated = dict(self)
+        for key, value in new_tags.items():
+            updated[key] = value
+
+        return frozentags(updated)
